// Ares, a tactical space combat game.
// Copyright (C) 1997, 1999-2001, 2008 Nathan Lamont
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#ifndef ANTARES_SOUND_FX_HPP_
#define ANTARES_SOUND_FX_HPP_

#include <stdint.h>
#include "MathSpecial.hpp"

namespace antares {

#define kSoundNum           48
#define kMaxChannelNum      3

#define kMaxVolumePreference    8

#define kMediumLoudVolume   192
#define kMediumVolume       128
#define kMediumLowVolume    64
#define kMaxSoundVolume     255         // minimum = 0

#define kShortPersistence       10      // in ticks
#define kMediumPersistence      20
#define kMediumLongPersistence  40
#define kLongPersistence        60

#define kPlayerBeamSound        500
#define kBasicExplosionSound    501
#define kPlayerPulseSound       502
#define kSmallHitSound          503
#define kAlienBeamSound         504
#define kAlienPulseSound        505
#define kMorseBeepSound         506 // ship receives order
#define kComputerBeep1          507 // ship selected
#define kComputerBeep2          508 // ship built
#define kComputerBeep3          509 // button push
#define kComputerBeep4          510 // change range
#define kWarningTone            511 // naughty beep
#define kHarshBeam              512
#define kLandingWoosh           513
#define kCloakOff               522
#define kCloakOn                523
#define kKlaxon                 525
#define kWarpOne                526
#define kWarpTwo                527
#define kWarpThree              528
#define kWarpFour               529
#define kTeletype               535

struct SndChannel;
struct Sound;
struct spaceObjectType;

enum soundPriorityType {
    kNoSound = 0,
    kVeryLowPrioritySound = 1,
    kLowPrioritySound = 2,
    kPrioritySound = 3,
    kHighPrioritySound = 4,
    kMustPlaySound = 5
};

struct smartSoundChannel {
    long                whichSound;
    long                soundAge;
    short               soundVolume;
    soundPriorityType   soundPriority;
    SndChannel*         channelPtr;
};

struct smartSoundHandle {
    sfz::scoped_ptr<Sound>   soundHandle;
    short               id;
    bool             keepMe;
};

void InitSoundFX();
void SetAllSoundsNoKeep();
void KeepSound(int sound_id);
int AddSound(int sound_id);
void RemoveAllUnusedSounds();
void ResetAllSounds();
void PlayVolumeSound(
        short whichSoundID, short amplitude, short persistence, soundPriorityType priority);
void PlayLocalizedSound(
        unsigned long sx, unsigned long sy, unsigned long dx, unsigned long dy,
        smallFixedType hvel, smallFixedType vvel, short whichSoundID, short amplitude,
        short persistence, soundPriorityType priority);
void SoundFXCleanup();

void mPlayDistanceSound(
        long mvolume, spaceObjectType* mobjectptr, long msoundid, long msoundpersistence,
        soundPriorityType msoundpriority);

}  // namespace antares

#endif // ANTARES_SOUND_FX_HPP_
